@@ -117,6 +117,7 @@ public:
     Addressee::List parseVCards(const QByteArray &vcard) const;
 
 private:
+    Q_DISABLE_COPY(VCardConverter)
     class VCardConverterPrivate;
     VCardConverterPrivate *const d;
 };
@@ -137,7 +138,7 @@ KCONTACTS_EXPORT QString dateToVCardString(const QDateTime &dateTime);
   * The return value is in the form "yyyyMMdd" (e.g. "20031201")
   * @param date date to be converted
   */
-KCONTACTS_EXPORT QString dateToVCardString(const QDate &date);
+KCONTACTS_EXPORT QString dateToVCardString(QDate date);
 
 /**
   * Converts a date string as it is used in VCard and LDIF files to a QDateTime value.
@@ -147,5 +148,11 @@ KCONTACTS_EXPORT QString dateToVCardString(const QDate &date);
   */
 KCONTACTS_EXPORT QDateTime VCardStringToDate(const QString &dateString);
 
+/**
+ * @brief adaptIMAttributes. Convert KAddressBook attribute to VCard IM Attribute
+ * @param data
+ */
+KCONTACTS_EXPORT void adaptIMAttributes(QByteArray &data);
+
 }
 #endif
