@@ -23,6 +23,8 @@
 #include <QtCore/QBuffer>
 #include <QtCore/QSharedData>
 
+#include <QtGui/QImage>
+
 namespace KContacts
 {
 
@@ -221,9 +223,7 @@ QString Picture::type() const
 
 QString Picture::toString() const
 {
-    QString str;
-
-    str += QLatin1String("Picture {\n");
+    QString str = QLatin1String("Picture {\n");
     str += QStringLiteral("  Type: %1\n").arg(d->mType);
     str += QStringLiteral("  IsIntern: %1\n").
            arg(d->mIntern ? QStringLiteral("true") : QStringLiteral("false"));
